@@ -38,6 +38,7 @@
 #include <ctype.h>
 #include <fcntl.h>
 #include <sys/types.h>
+#include <sys/resource.h>
 #include <sys/stat.h>
 #include <signal.h>
 #include <pwd.h>
@@ -150,6 +151,7 @@ static gint GdmRetryDelay = 0;
 static gint GdmTimedLoginDelay = 0;
 static gint GdmFlexibleXservers = 5;
 static gint GdmFirstVt = 7;
+static gint GdmXserverTimeout = 10;
 
 /* The SDTLOGIN feature is Solaris specific, and causes the Xserver to be
  * run with user permissionsinstead of as root, which adds security but
@@ -532,6 +534,7 @@ gdm_config_init (void)
       &GdmBackgroundProgramInitialDelay, &int_type);
    gdm_config_add_hash (GDM_KEY_BACKGROUND_PROGRAM_RESTART_DELAY,
       &GdmBackgroundProgramRestartDelay, &int_type);
+   gdm_config_add_hash (GDM_KEY_XSERVER_TIMEOUT, &GdmXserverTimeout, &int_type);
 }
 
 /**
@@ -575,9 +578,28 @@ gdm_get_default_config (struct stat *statbuf)
 static VeConfig *
 gdm_get_custom_config (struct stat *statbuf)
 {
-   VeConfig *retval;
    int r;
 
+   /*
+    * First check to see if the old configuration file name is on
+    * the system.  If so, use that as the custom configuration
+    * file.  "make install" will move this file aside, and 
+    * distros probably can also manage moving this file on
+    * upgrade.   
+    *
+    * In case this file is on the system, then use it as
+    * the custom configuration file until the user moves it
+    * aside.  This will likely mean all the defaults in
+    * defaults.conf will not get used since the old gdm.conf
+    * file has all the keys in it (except new ones).  But
+    * that would be what the user wants.
+    */
+   VE_IGNORE_EINTR (r = g_stat (GDM_OLD_CONF, statbuf));
+   if (r >= 0) {
+      custom_config_file = g_strdup (GDM_OLD_CONF);
+      return ve_config_new (custom_config_file);
+   }
+
    VE_IGNORE_EINTR (r = g_stat (GDM_CUSTOM_CONF, statbuf));
    if (r >= 0) {
       custom_config_file = g_strdup (GDM_CUSTOM_CONF);
@@ -587,6 +609,17 @@ gdm_get_custom_config (struct stat *statbuf)
    }
 }
 
+/**
+ * gdm_get_custom_config_file
+ *
+ * Returns the custom config file being used.
+ */
+gchar *
+gdm_get_custom_config_file (void)
+{
+   return custom_config_file;
+}
+
 /**
  * gdm_get_value_int
  *
@@ -686,7 +719,6 @@ gdm_config_to_string (gchar *key, gchar **retval)
    if (translated_hash != NULL) {
       gpointer val = gdm_config_hash_lookup (translated_hash, key);
       if (val) {
-         gchar *charval = (char *)val;
          *retval = g_strdup (val);
          return;
       }
@@ -853,8 +885,8 @@ _gdm_set_value_string (gchar *key, gchar *value_in, gboolean doing_update)
        } else {
           gdm_info (_("%s: BaseXsession empty; using %s/gdm/Xsession"),
                       "gdm_config_parse",
-                      EXPANDED_SYSCONFDIR);
-          *setting = g_build_filename (EXPANDED_SYSCONFDIR,
+                      GDMCONFDIR);
+          *setting = g_build_filename (GDMCONFDIR,
                                        "gdm", "Xsession", NULL);
        }
 
@@ -1141,6 +1173,8 @@ static gboolean
 gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_update) 
 {
    gchar * realkey = gdm_config_hash_lookup (realkey_hash, key);
+   gchar *value;
+
    if (realkey == NULL) {
       return FALSE;
    }
@@ -1180,12 +1214,13 @@ gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_up
           while (list != NULL) {
              if (g_str_has_prefix ((char *)list->data, prefix) &&
                  g_str_has_suffix ((char *)list->data, "]")) {
+                gchar *transkey, *transvalue;
 
                 if (translated_hash == NULL)
                    translated_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-                gchar    *transkey     = g_strdup_printf ("greeter/%s", (char *)list->data);
-                gchar    *transvalue   = ve_config_get_string (cfg, transkey);
+                transkey   = g_strdup_printf ("greeter/%s", (char *)list->data);
+                transvalue = ve_config_get_string (cfg, transkey);
 
                 g_hash_table_remove (translated_hash, transkey);
 
@@ -1204,7 +1239,7 @@ gdm_set_value (VeConfig *cfg, GdmConfigType *type, gchar *key, gboolean doing_up
        }
 
        /* Handle non-translated strings as normal */
-       gchar *value = ve_config_get_string (cfg, realkey);
+       value = ve_config_get_string (cfg, realkey);
        _gdm_set_value_string (key, value, doing_update);
        return TRUE;
    }
@@ -1281,7 +1316,6 @@ static void
 gdm_load_xservers (VeConfig *cfg)
 {
    GList *list, *li;
-   GSList *xli;
 
    /* Find server definitions */
    list = ve_config_get_sections (cfg);
@@ -1548,30 +1582,36 @@ check_servauthdir (struct stat *statbuf)
     /* Enter paranoia mode */
     VE_IGNORE_EINTR (r = g_stat (GdmServAuthDir, statbuf));
     if G_UNLIKELY (r < 0) {
-            gchar *s = g_strdup_printf
-                    (C_(N_("Server Authorization directory "
-                           "(daemon/ServAuthDir) is set to %s "
-                           "but this does not exist. Please "
-                           "correct GDM configuration and "
-                           "restart GDM.")), GdmServAuthDir);
-        if (GdmConsoleNotify)
-                    gdm_text_message_dialog (s);
+            if (GdmConsoleNotify) {
+               gchar *s = g_strdup_printf
+                       (C_(N_("Server Authorization directory "
+                              "(daemon/ServAuthDir) is set to %s "
+                              "but this does not exist. Please "
+                              "correct GDM configuration and "
+                              "restart GDM.")), GdmServAuthDir);
+
+               gdm_text_message_dialog (s);
+               g_free (s);
+            }
+
             GdmPidFile = NULL;
-            g_free (s);
             gdm_fail (_("%s: Authdir %s does not exist. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 
     if G_UNLIKELY (! S_ISDIR (statbuf->st_mode)) {
-            gchar *s = g_strdup_printf
-                    (C_(N_("Server Authorization directory "
-                           "(daemon/ServAuthDir) is set to %s "
-                           "but this is not a directory. Please "
-                           "correct GDM configuration and "
-                           "restart GDM.")), GdmServAuthDir);
-        if (GdmConsoleNotify)
-                    gdm_text_message_dialog (s);
+            if (GdmConsoleNotify) {
+               gchar *s = g_strdup_printf
+                       (C_(N_("Server Authorization directory "
+                              "(daemon/ServAuthDir) is set to %s "
+                              "but this is not a directory. Please "
+                              "correct GDM configuration and "
+                              "restart GDM.")), GdmServAuthDir);
+
+                gdm_text_message_dialog (s);
+                g_free (s);
+            }
+
             GdmPidFile = NULL;
-            g_free (s);
             gdm_fail (_("%s: Authdir %s is not a directory. Aborting."), "gdm_config_parse", GdmServAuthDir);
     }
 }
@@ -1661,7 +1701,6 @@ gdm_load_config_option (gpointer key_in, gpointer value_in, gpointer data)
    gchar *key               = (gchar *)key_in;
    GdmConfigType *type      = (GdmConfigType *)value_in;
    GdmConfigFiles *cfgfiles = (GdmConfigFiles *)data;
-   gboolean retval;
    gboolean custom_retval;
 
    if (type != NULL) {
@@ -1798,14 +1837,17 @@ gdm_config_parse (void)
          GdmAutomaticLogin = NULL;
          GdmTimedLogin     = NULL;
       } else {
-         gchar *s = g_strdup_printf (C_(N_("XDMCP is disabled and GDM "
-                                   "cannot find any static server "
-                                   "to start.  Aborting!  Please "
-                                   "correct the configuration "
-                                   "and restart GDM.")));
-         gdm_text_message_dialog (s);
+         if (GdmConsoleNotify) {
+            gchar *s = g_strdup_printf (C_(N_("XDMCP is disabled and GDM "
+                                      "cannot find any static server "
+                                      "to start.  Aborting!  Please "
+                                      "correct the configuration "
+                                      "and restart GDM.")));
+            gdm_text_message_dialog (s);
+            g_free (s);
+         }
+
          GdmPidFile = NULL;
-         g_free (s);
          gdm_fail (_("%s: XDMCP disabled and no static servers defined. Aborting!"), "gdm_config_parse");
       }
    }
@@ -1823,57 +1865,66 @@ gdm_config_parse (void)
 
    /* Set GdmUserId and GdmGroupId */
    if G_UNLIKELY (pwent == NULL) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM user '%s' does not exist. "
-                                        "Please correct GDM configuration "
-                                        "and restart GDM.")), GdmUser);
-      if (GdmConsoleNotify)
+
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM user '%s' does not exist. "
+                                           "Please correct GDM configuration "
+                                           "and restart GDM.")), GdmUser);
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
 
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Can't find the GDM user '%s'. Aborting!"), "gdm_config_parse", GdmUser);
    } else {
       GdmUserId = pwent->pw_uid;
    }
 
    if G_UNLIKELY (GdmUserId == 0) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM user is set to be root, but "
-                                        "this is not allowed since it can "
-                                        "pose a security risk.  Please "
-                                        "correct GDM configuration and "
-                                        "restart GDM.")));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM user is set to be root, but "
+                                           "this is not allowed since it can "
+                                           "pose a security risk.  Please "
+                                           "correct GDM configuration and "
+                                           "restart GDM.")));
+
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: The GDM user should not be root. Aborting!"), "gdm_config_parse");
    }
 
    grent = getgrnam (GdmGroup);
 
    if G_UNLIKELY (grent == NULL) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM group '%s' does not exist. "
-                                        "Please correct GDM configuration "
-                                        "and restart GDM.")), GdmGroup);
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM group '%s' does not exist. "
+                                           "Please correct GDM configuration "
+                                           "and restart GDM.")), GdmGroup);
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Can't find the GDM group '%s'. Aborting!"), "gdm_config_parse", GdmGroup);
    } else  {
       GdmGroupId = grent->gr_gid;   
    }
 
    if G_UNLIKELY (GdmGroupId == 0) {
-      gchar *s = g_strdup_printf (C_(N_("The GDM group is set to be root, but "
-                                        "this is not allowed since it can "
-                                        "pose a security risk. Please "
-                                        "correct GDM configuration and "
-                                        "restart GDM.")));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("The GDM group is set to be root, but "
+                                           "this is not allowed since it can "
+                                           "pose a security risk. Please "
+                                           "correct GDM configuration and "
+                                           "restart GDM.")));
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: The GDM group should not be root. Aborting!"), "gdm_config_parse");
    }
 
@@ -1904,9 +1955,10 @@ gdm_config_parse (void)
 
    /* Check the serv auth and log dirs */
    if G_UNLIKELY (ve_string_empty (GdmServAuthDir)) {
-       if (GdmConsoleNotify)
+       if (GdmConsoleNotify) {
           gdm_text_message_dialog
              (C_(N_("No daemon/ServAuthDir specified in the GDM configuration file")));
+       }
        GdmPidFile = NULL;
        gdm_fail (_("%s: No daemon/ServAuthDir specified."), "gdm_config_parse");
    }
@@ -1931,33 +1983,38 @@ gdm_config_parse (void)
    check_servauthdir (&statbuf);
 
    if G_UNLIKELY (statbuf.st_uid != 0 || statbuf.st_gid != GdmGroupId)  {
-      gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
-                                        "(daemon/ServAuthDir) is set to %s "
-                                        "but is not owned by user %s and group "
-                                        "%s. Please correct the ownership or "
-                                        "GDM configuration and restart "
-                                        "GDM.")), GdmServAuthDir, GdmUser, GdmGroup);
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
+                                           "(daemon/ServAuthDir) is set to %s "
+                                           "but is not owned by user %s and group "
+                                           "%s. Please correct the ownership or "
+                                           "GDM configuration and restart "
+                                           "GDM.")), GdmServAuthDir,
+		   			gdm_root_user (), GdmGroup);
          gdm_text_message_dialog (s);
-         GdmPidFile = NULL;
          g_free (s);
-         gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."),
+      }
+
+      GdmPidFile = NULL;
+      gdm_fail (_("%s: Authdir %s is not owned by user %s, group %s. Aborting."),
                      "gdm_config_parse", GdmServAuthDir, gdm_root_user (), GdmGroup);
    }
 
    if G_UNLIKELY (statbuf.st_mode != (S_IFDIR|S_IRWXU|S_IRWXG|S_ISVTX))  {
-      gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
-                                 "(daemon/ServAuthDir) is set to %s "
-                                 "but has the wrong permissions: it "
-                                 "should have permissions of %o. "
-                                 "Please correct the permissions or "
-                                 "the GDM configuration and "
-                                 "restart GDM.")), GdmServAuthDir,
-                                 (S_IRWXU|S_IRWXG|S_ISVTX));
-      if (GdmConsoleNotify)
+      if (GdmConsoleNotify) {
+         gchar *s = g_strdup_printf (C_(N_("Server Authorization directory "
+                                    "(daemon/ServAuthDir) is set to %s "
+                                    "but has the wrong permissions: it "
+                                    "should have permissions of %o. "
+                                    "Please correct the permissions or "
+                                    "the GDM configuration and "
+                                    "restart GDM.")), GdmServAuthDir,
+                                    (S_IRWXU|S_IRWXG|S_ISVTX));
          gdm_text_message_dialog (s);
+         g_free (s);
+      }
+
       GdmPidFile = NULL;
-      g_free (s);
       gdm_fail (_("%s: Authdir %s has wrong permissions %o. Should be %o. Aborting."), "gdm_config_parse", 
                   GdmServAuthDir, statbuf.st_mode, (S_IRWXU|S_IRWXG|S_ISVTX));
    }
@@ -2139,7 +2196,7 @@ static gboolean
 is_in_trusted_pic_dir (const char *path)
 {
         /* our own pixmap dir is trusted */
-        if (strncmp (path, EXPANDED_PIXMAPDIR, sizeof (EXPANDED_PIXMAPDIR)) == 0)
+        if (strncmp (path, PIXMAPDIR, sizeof (PIXMAPDIR)) == 0)
                 return TRUE;
 
         return FALSE;
