@@ -20,7 +20,6 @@
 
 #include "config.h"
 
-#include <syslog.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -31,7 +30,7 @@
 #include <math.h>
 #include <string.h>
 #include <gtk/gtk.h>
-#include <libgnome/libgnome.h>
+#include <glib/gi18n.h>
 #include <librsvg/rsvg.h>
 
 #include "gdm.h"
@@ -70,6 +69,10 @@ check_for_displays (void)
 	char  *auth_cookie = NULL;
 	int    i;
 
+	/*
+	 * Might be nice to move this call into read_config() so that it happens
+	 * on the same socket call as reading the configuration.
+	 */
 	ret = gdmcomm_call_gdm (GDM_SUP_ATTACHED_SERVERS, auth_cookie, "2.2.4.0", 5);
 	if (ve_string_empty (ret) || strncmp (ret, "OK ", 3) != 0) {
 		g_free (ret);
@@ -114,8 +117,7 @@ gdm_greeter_users_init (void)
 				       gdm_config_get_int (GDM_KEY_MAX_ICON_WIDTH),
 				       gdm_config_get_int (GDM_KEY_MAX_ICON_HEIGHT));
 	if (! defface) {
-		syslog (LOG_WARNING,
-			_("Can't open DefaultImage: %s!"),
+		gdm_common_warning ("Can't open DefaultImage: %s!",
 			gdm_config_get_string (GDM_KEY_DEFAULT_FACE));
 	}
 
@@ -180,19 +182,17 @@ user_selected (GtkTreeSelection *selection, gpointer data)
 		gtk_tree_model_get (tm, &iter, GREETER_ULIST_LOGIN_COLUMN,
 				    &login, -1);
 		if (login != NULL) {
-			GreeterItemInfo *pamlabel;
-
 			if (selecting_user && greeter_probably_login_prompt) {
 				gtk_entry_set_text (GTK_ENTRY (pam_entry), login);
 			}
-			pamlabel = greeter_lookup_id ("pam-message");
-			if (selecting_user && pamlabel != NULL) {
+			if (selecting_user) {
+				GreeterItemInfo *pamlabel = greeter_lookup_id ("pam-message");
+				if (pamlabel == NULL) {
+					gdm_common_warning ("Theme broken: must have pam-message label!");
+				}
 				printf ("%c%c%c%s\n", STX, BEL,
 					GDM_INTERRUPT_SELECT_USER, login);
 				fflush (stdout);
-			} else {
-				syslog (LOG_WARNING,
-					_("Theme broken: must have pam-message label!"));
 			}
 		}
 	}
